/*
    Copyright 2008-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/
#ifndef EMEA_P_H
#define EMEA_P_H

#include <datasetchain.h>

class EMEAFinishedWatcher : public NSCore::DataSetChainMember
{
	Q_OBJECT
	public:
		EMEAFinishedWatcher( QObject* p )
			: NSCore::DataSetChainMember( p )
			, newDatasetsArrived( true )
			, timer( new QTimer( this ) )
			, interval( 1000*60* 6 )
		{
			qDebug() << "EMEAFinishedWatcher::EMEAFinishedWatcher(" << p << ")";
			timer->setInterval( interval );
			connect( timer, SIGNAL( timeout() ), this, SLOT( checkTime() ) );
		}

		void start() {
			timer->start();
		}

		void setInterval( int n ) {
			interval = n;
			timer->setInterval( interval );
		}

	protected:
		bool dataArrived( NSCore::AcqisitionDataSet& ) {
			newDatasetsArrived = true;
			return true;
		}

	signals:
		void evaluationFinished();

	private slots:
		void checkTime() {
			if ( ! newDatasetsArrived ) {
				qDebug() << "EMEAFinishedWatcher::checkTime() will emit the finished signal now.";
				emit evaluationFinished();
				timer->stop();
			}
			newDatasetsArrived = false;
		}
	private:
		bool newDatasetsArrived;
		QTimer* timer;
		int interval;

		QString debug() const { return "EMEAFinishedWatcher"; }
		QDomNode saveToXML( QDomDocument ) { return QDomNode(); }
		bool restoreFromXML( QDomNode ) { return false; }
};
#endif // EMEA_P_H
