/*
    Copyright 2008-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "emea.h"

#include <QtGui/QApplication>
#include <QtCore/QDebug>

#include <acqiris.h>
#include <resulttospectrum.h>
#include <evaluationpath.h>

class EMEAAcqiris : public EMEAWindow
{
	public:
		EMEAAcqiris( QWidget* p ) : EMEAWindow( p ) {
			qDebug() << "EMEAAcqiris::EMEAAcqiris(" << p << ")";
		}

		void setupChain( int threads ) {
			qDebug() << "EMEAAcqiris::setupChain(" << threads << ")";
			acqiris = objectInNewThread<NSAcqiris::Acqiris>();
			for ( int i=0; i<acqiris->digitizers(); ++i ) {
				qDebug() << i << "got digitizer" << acqiris->digitizer( i );
				addToChainStart( acqiris->digitizer( i ) );
				acqiris->digitizer( i )->setDispatching( NSCore::DataSetProducer::DispatchEqual );
			}
			QList<NSCore::DataSetChainMember*> evals;
			for ( int i=0; i<threads; ++i )
				evals << objectInNewThread<NSEvaluation::EvaluationPath>();
			addToChain( evals );
			ResultsToSpectrum* res = objectInNewThread<ResultsToSpectrum>();
			connect( res, SIGNAL( newSpectrum( AbstractSpectrumModel* ) ), spectrumGuiCreator(), SLOT( newSpectrum( AbstractSpectrumModel* ) ) );
			addToChain( res );
		}

		NSAcqiris::Acqiris* acqiris;

		void autoStart() {
			qDebug() << "EMEAAcqiris::autoStart()";
			for ( int i=0; i<acqiris->digitizers(); ++i )
				acqiris->digitizer( i )->startAcqisition();
		}
};

int main( int argc, char** argv )
{

	QApplication app( argc, argv );

	EMEAWindow* w = new EMEAAcqiris( 0 );
	w->show();

	int ret = app.exec();

	delete w;

	return ret;
};

