/*
    Copyright 2008-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/

#include "valuelogger.h"
#include "valuelogger.moc"

#include <QtCore/QTimer>
#include <QtCore/QDebug>

#include <QtGui>

Logger::Logger( AbstractLogging* p ) : QObject( 0 ) {
	//setParent( dynamic_cast<QObject*>( p ) );
}
void Logger::logValue( double n ) {
	_value += n;
	_counter++;
}
double Logger::getValue() {
	double ret;
	if ( _counter == 0 )
		ret = 0;
	else
		ret = _value / _counter;
	_counter = 0;
	_value = 0;
	return ret;
}

AbstractLogging::AbstractLogging() {
	qDebug() << "AbstractLogging::AbstractLogging()";
}
Logger* AbstractLogging::getLogger( QString n ) {
	if ( ! _loggers.contains( n ) ) {
		Logger* tmp = new Logger( this );
		_loggers[ n ] = tmp;
	}
	return _loggers[ n ];
}
void AbstractLogging::addLogger( QString n, QObject* obj, const char* signal ) {
	QObject::connect( obj, signal, getLogger( n ), SLOT( logValue( double ) ) );
}
AbstractLogging::logvalues_t AbstractLogging::getLogValues() {
	AbstractLogging::logvalues_t ret;
	foreach( QString logger, _loggers.keys() ) {
		ret[ logger ] = _loggers[ logger ]->getValue();
	}
	return ret;
}

ValueLogging::ValueLogging( QObject* p ) : QObject( p ) {
	QTimer* timer = new QTimer( this );
	timer->setInterval( 1000*60 );
	connect( timer, SIGNAL( timeout() ), this, SLOT( loggingTimeout() ) );
	timer->start();
}
void ValueLogging::loggingTimeout() {
	QString ret;
	ValueLogging::logvalues_t tmp = getLogValues();
	foreach( QString logger, tmp.keys() ) {
		ret += QString( "%1 %2; " ).arg( logger ).arg( tmp[ logger ] );
	}
	qDebug() << ret;
}

GuiLogging::GuiLogging( QWidget* p ) : QWidget( p ) {
	setObjectName( "Countrates" );
	qDebug() << "GuiLogging::GuiLogging(" << p << ")";
	QTimer* timer = new QTimer( this );
	timer->setInterval( 1000*2 );
	connect( timer, SIGNAL( timeout() ), this, SLOT( loggingTimeout() ) );
	timer->start();

	new QGridLayout( this );
}
GuiLogging::~GuiLogging() {
	qDebug() << "GuiLogging::~GuiLogging()";
}
void GuiLogging::loggingTimeout() {
	//qDebug() << "GuiLogging::loggingTimeout()";
	logvalues_t values = getLogValues();
	foreach ( QString logger, values.keys() ) {
		//qDebug() << " logger\t:" << logger;
		GuiLoggingView* view = _views[ logger ];
		//qDebug() << " view\t:" << view;
		if ( view == 0 ) {
			view = new GuiLoggingView( this );
			layout()->addWidget( view );
			_views[ logger ] = view;
		}
		view->addValue( values[ logger ] );
	}
}

GuiLoggingView::GuiLoggingView( GuiLogging* p ) : QGraphicsView( p ) {
	setAlignment( Qt::AlignBottom | Qt::AlignRight );
	_scene = new QGraphicsScene( this );
	setScene( _scene );
	_group = _scene->createItemGroup( QList<QGraphicsItem*>()/* << _scene->addLine( 1,-1, 1,0 )*/ );
	connect( _scene, SIGNAL( sceneRectChanged( const QRectF& ) ), this, SLOT( autoZoom( const QRectF& ) ) );
}

void GuiLoggingView::addValue( double n ) {
	_group->moveBy( -0.5,0 );
	QGraphicsLineItem* line = _scene->addLine( 0,-1, 0,n );
	_group->addToGroup( line );
}

void GuiLoggingView::autoZoom( const QRectF& rect ) {
	QMatrix m;
	if ( rect.height() > 10 && rect.height() != viewport()->height() ) {
		m.scale( 1, - viewport()->height() / rect.height() );
	}
	if ( rect.width() > viewport()->width() ) {
		m.scale( viewport()->width() / rect.width(), 1);
	}
	setMatrix( m );
}

