/*
    Copyright 2007-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>
                        Nicki Hinsche <nicvok@freenet.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DOPPLERPLUGINS_H
#define DOPPLERPLUGINS_H

#include "plugins_interface.h"

//#include "interpolation_helpers.h"

class TimeConstantAnalyzer : public NSEvaluation::Analyzer
{
	public:
		TimeConstantAnalyzer(  NSCore::QTreeItem* );

		void analyze(  NSCore::AcqisitionDataSet& data );

		QString type() const { return "TimeConstantAnalyzer"; }

	private:
		double lin_regression(  int x_max, double*, int n_0 ) const;
};

class IntegrationAnalyzerCF : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzerCF( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzerCF"; }

	private:
		double check_cf( double lfp, double hfp, int x_max, int x_min, double*, int n_0 ) const;
		double check_cfV2( double lfp, double hfp, int x_min, int x_max, double mean, double deviation, double* ch, int n_0 ) const;
};

class IntegrationAnalyzerCFspline : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzerCFspline( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzerCFspline"; }

	private:
		double cfarnold( double x_frac, int x_max, int x_min, double y_max, double y_min, double* ch, int n_0 ) const;
};

class IntegrationAnalyzer : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzer"; }

	private:
		double sumup( int x_max, int x_min, double*, int n_0 ) const;
	
};

class IntegrationAnalyzerZeroLine : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzerZeroLine( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzerZeroLine"; }

	private:
		double IntegrateZeroline( double x_frac, int x_max, int x_min, double y_max, double y_min, double zeroline, int select, double* ch, int n_0 ) const;
	
};

class IntegrationAnalyzerDiff : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzerDiff( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzerDiff"; }

	private:
		double diff( int frac_at, double difference, int x_max, int x_min, double y_max, double y_min, double* ch, int n_0 ) const;
	
};



class IntegrationAnalyzerAnalytic : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzerAnalytic( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzerAnalytic"; }

	private:
		
		double IntegrateAnalytic( double lfp, double hfp, int x_max, int x_min, double y_max, double y_min, double zeroline, double* ch, int n_0 ) const;
	
};





class TimeConstantIntegrationAnalyzer : public NSEvaluation::Analyzer
{
	public:
		TimeConstantIntegrationAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "TimeConstantIntegrationAnalyzer"; }

	private:
		double lin_regression_integrate( int xfrac_at, double xzero, int xmax_at, int x_dist_reg, int x_dist_int, int x_lenght, double* ch, int n_0 ) const;
	
};



/*class CFAnalyzer : public NSEvaluation::Analyzer, private PolynomInterpolate {
	public:
		CFAnalyzer( NSCore::QTreeItem* );
		~CFAnalyzer();

		QString type() const { return "CFAnalyzer"; }

		void analyze( NSCore::AcqisitionDataSet& data );

	private:
		coord points[ 3 ];
};*/

#endif // DOPPLERPLUGINS_H

