/*
    Copyright 2007-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>
                        Nicki Hinsche <nicvok@freenet.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "gagereader.h"
#include "gagereader.moc"

#include <QtCore/QDataStream>
#include <math.h>

GageReader::GageReader( QObject* p )
	: NSDiskIO::AbstractReader( p )
	, NSDiskIO::AbstractDiskIO<QDataStream>( QIODevice::ReadOnly )
{
	setObjectName( "GageReader" );
}
GageReader::~GageReader() {
}
void GageReader::setFile( QString file ) {
	NSDiskIO::AbstractDiskIO<QDataStream>::setFile( file );
}
QString GageReader::filename() const {
	return NSDiskIO::AbstractDiskIO<QDataStream>::filename();
}

void GageReader::readSingleDataSet() {

	QVector<double> datavector;//if size is fixed reading is faster,
	int skip=512;
	qint16 buffer=0;

	_stream->skipRawData(skip); //ignore "skip bytes" from the beginning

	_stream->setByteOrder( QDataStream::LittleEndian );

	while ( ! _stream->atEnd() ) {
		*_stream >> buffer;
		datavector.push_back( buffer );	
	}


	int const channels = 1;

	NSCore::AcqisitionDataSet data( channels/*channel*/, (datavector.size()) /*samples*/ );///check this

	double* ch = data.getRawChannel( 0 );
	const double imax = pow( 2,14 );
	for ( int i=0; i<data.samples(); ++i ) {
		ch[ i ] = datavector[ i ] / imax;
	}
	data.updateMinMax();

	double range = data.result( "channel 0/max" )->toDouble() - qMin( data.result( "channel 0/min" )->toDouble(), 0.0 );
	data.addResult( "channel 0/fullscale", range*2 );

	enqueue( data );

	//QString newnewname = NewName( data.channel(0), filename );
	//setFile(newnewname);


	//for type of file that follows the structure: directory_channelcount/foldername.foldercount/filename-channelcount-filecount.suffix



	QString cutsuffix=filename().section('.',0,-2);   	//cut off suffix
	QString cutfilecount=cutsuffix.section('-',0,-2);	//cut off filecount
	QString cutfoldercount=cutfilecount.section('.',0,-2);	//cut off inclusive foldercount

	QString step=cutfilecount.section('.',-1,-1);
	QString foldercountstring=step.section('/',-2,-2);
	QString filecountstring = cutsuffix.section( '-', -1, -1 );

	QString cutfilename=cutfilecount.section('/',-1,-1);
	QString cutname=cutfilename.section('_',0,-1).remove(-2,4);
	QString cutfoldername=cutfilecount.section('.',-2,-2) + ".";
	QString cutfoldernamewochannel=cutfilecount.section('.',-2,-2).section('_',0,-2) + "_CH";
	QString channelcountstring=cutfoldername.section('_',-1,-1).section('/',-2,-2).remove(0,2);

	int filecount = filecountstring.toInt();
	//int channelcount = channelcountstring.toInt();




	QString newfilecountstring = QString::number( filecount+1 );
	while ( newfilecountstring.size() < filecountstring.size() )
		newfilecountstring.prepend( "0" );

	QString newname = cutfoldernamewochannel + channelcountstring + "/Folder." + foldercountstring + "/" + cutfilename + "-" + newfilecountstring + ".sig";
	//QString newname2 = cutfoldernamewochannel + "02" + "/Folder." + foldercountstring + "/" + cutname + "02" + "-" + newfilecountstring + ".sig";

	if ( ! QFile::exists( newname ) ) {
		int foldercount = foldercountstring.toInt();

		QString newfoldercount = QString::number( foldercount+1 );
		while ( newfoldercount.size() < foldercountstring.size() )
			newfoldercount.prepend( "0" );

		newname = cutfoldernamewochannel + channelcountstring + "/Folder." + newfoldercount + "/" + cutfilename + "-" + newfilecountstring + ".sig";
		//qDebug() << "new name would be: " << newname; 
	}


	setFile( newname );

};




///function for finding new filename

/*QString GageReader::NewName(NSCore::ChannelDataSet* ch , QString filename() ) const {


	QString cutsuffix=filename().section('.',0,-2);   	//cut off suffix
	QString cutfilecount=cutsuffix.section('-',0,-2);	//cut off filecount
	QString cutfoldercount=cutfilecount.section('.',0,-2);	//cut off inclusive foldercount

	QString step=cutfilecount.section('.',-1,-1);
	QString foldercountstring=step.section('/',-2,-2);
	QString filecountstring = cutsuffix.section( '-', -1, -1 );

	QString cutfilename=cutfilecount.section('/',-1,-1);
	QString cutname=cutfilename.section('_',0,-1).remove(-2,4);
	QString cutfoldername=cutfilecount.section('.',-2,-2) + ".";
	QString cutfoldernamewochannel=cutfilecount.section('.',-2,-2).section('_',0,-2) + "_CH";
	QString channelcountstring=cutfoldername.section('_',-1,-1).section('/',-2,-2).remove(0,2);

	int filecount = filecountstring.toInt();
	

	QString newfilecountstring = QString::number( filecount+1 );
	while ( newfilecountstring.size() < filecountstring.size() )
		newfilecountstring.prepend( "0" );

	QString newname = cutfoldernamewochannel + channelcountstring + "/Folder." + foldercountstring + "/" + cutfilename + "-" + newfilecountstring + ".sig";
	QString newname2 = cutfoldernamewochannel + "02" + "/Folder." + foldercountstring + "/" + cutname + "02" + "-" + newfilecountstring + ".sig";


	if ( ! QFile::exists( newname ) ) {
		//qDebug() << " ! File does not exist!";
		int foldercount = foldercountstring.toInt();

		QString newfoldercount = QString::number( foldercount+1 );
		while ( newfoldercount.size() < foldercountstring.size() )
			newfoldercount.prepend( "0" );

		newname = cutfoldernamewochannel + channelcountstring + "/Folder." + newfoldercount + "/" + cutfilename + "-" + newfilecountstring + ".sig";
		//qDebug() << "new name would be: " << newname; 
	}

	if(ch=0)
	QString name = newname ;
	else
	QString name = newname2 ;

	qDebug() << " New name1 would be" << newname;
	qDebug() << " New name2 would be" << newname2;


	return name;

}*/



///test for parallel reading of two channels

/*

///nescessary reading n datasheets?

void GageReader::generateNDataSets( int n ) {
	if ( _togenerate == 0 )
		QTimer::singleShot( 10, this, SLOT( generateDataSet() ) );
	_togenerate += n;
}

///this one is evident

void GageReader::generateDataSet() {

	if ( QThread::currentThread() != thread() ) {
		QTimer::singleShot( 100, this, SLOT( generateDataSet() ) );///whats the 100
		return;
	}


	const int channels = 2; ///for two channels

	AcqisitionDataSet* data = new AcqisitionDataSet( channels, _samples );

	///loop for the channels

	for ( int i=0; i<data->channels(); ++i ) {

		ChannelDataSet* channel = data->channel( i );//choosen channel is linked

		///setfile if i=0 -> as newname else if i=1 -> newname2 else error

		QString cutsuffix=filename().section('.',0,-2);   	//cut off suffix
		QString cutfilecount=cutsuffix.section('-',0,-2);	//cut off filecount
		QString cutfoldercount=cutfilecount.section('.',0,-2);	//cut off inclusive foldercount

		QString step=cutfilecount.section('.',-1,-1);
		QString foldercountstring=step.section('/',-2,-2);
		QString filecountstring = cutsuffix.section( '-', -1, -1 );
	
		QString cutfilename=cutfilecount.section('/',-1,-1);
		QString cutname=cutfilename.section('_',0,-1).remove(-2,4);
		QString cutfoldername=cutfilecount.section('.',-2,-2) + ".";
		QString cutfoldernamewochannel=cutfilecount.section('.',-2,-2).section('_',0,-2) + "_CH";
		QString channelcountstring=cutfoldername.section('_',-1,-1).section('/',-2,-2).remove(0,2);

		int filecount = filecountstring.toInt();
		//int channelcount = channelcountstring.toInt();
	
	
		//qDebug() << channelcountstring << channelcount; //01 and 1
		//qDebug() << cutfilename;
		//qDebug() << cutname;


		QString newfilecountstring = QString::number( filecount+1 );
		while ( newfilecountstring.size() < filecountstring.size() )
			newfilecountstring.prepend( "0" );
	
		QString newname = cutfoldernamewochannel + channelcountstring + "/Folder." + foldercountstring + "/" + cutfilename + "-" + newfilecountstring + ".sig";
		QString newname2 = cutfoldernamewochannel + "02" + "/Folder." + foldercountstring + "/" + cutname + "02" + "-" + newfilecountstring + ".sig";
	
		qDebug() << " New name1 would be" << newname;
		qDebug() << " New name2 would be" << newname2;

		if ( ! QFile::exists( newname ) ) {
			//qDebug() << " ! File does not exist!";
			int foldercount = foldercountstring.toInt();
	
			QString newfoldercount = QString::number( foldercount+1 );
			while ( newfoldercount.size() < foldercountstring.size() )
				newfoldercount.prepend( "0" );
	
			newname = cutfoldernamewochannel + channelcountstring + "/Folder." + newfoldercount + "/" + cutfilename + "-" + newfilecountstring + ".sig";
			//qDebug() << "new name would be: " << newname; 
	}

	if(i=0)
	setFile( newname );
	else
	setfile( newname1 ); 


		///loop for the single sample / data

		
		const double imax = pow( 2,14 );
		for ( int j=0;j<data.channel( i )->samples(); ++j ) {
		data.channel( i )->setSample( j, datavector[ j ] / imax );
		}
		
		double range = channel->result( "max" )->toDouble() - qMin( channel->result( "min" )->toDouble(), 0.0 );
		channel->parameter( "fullscale" )->setValue( range*2 );
	}

	enqueue( data );
	signalData();

	--_togenerate;
	if ( _togenerate > 0 ) {
		if ( _ownthread )
			QTimer::singleShot( 0, this, SLOT( generateDataSet() ) );
		else
			QTimer::singleShot( 10, this, SLOT( generateDataSet() ) );
	}
}*/

///Appendix for counting channels up

	/*if(j=0)
	setFile( newname );
	else
	setFile( newname2 );*/

	//qDebug() << " New name1 would be" << newname;
	//qDebug() << " New name2 would be" << newname2;

	//setFile( newname);

	/*if ( ! QFile::exists( newname ) ) {
		//qDebug() << " ! File does not exist!";
		//int foldercount = foldercountstring.toInt();
		
		QString newchannelcount = QString::number( channelcount+1 );//need to implement channelcount
		while ( newchannelcount.size() < channelcountstring.size() )
			newchannelcount.prepend( "0" );

		int filecount=0;
		int foldercount=0;

		QString newfilecountstring = QString::number( filecount+1 );
		while ( newfilecountstring.size() < filecountstring.size() )
		newfilecountstring.prepend( "0" );	
	
		QString newfoldercount = QString::number( foldercount+1 );
		while ( newfoldercount.size() < foldercountstring.size() )
			newfoldercount.prepend( "0" );
		
		newname = cutfoldernamewochannel + newchannelcount + "/Folder." + newfoldercount + "/" + cutfilename + "-" + newfilecountstring + ".sig";

	}*/
