/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef EVALUATION_PATH_H
#define EVALUATION_PATH_H

#include <QtCore/QObject>
#include <QtCore/QDebug>
#include <QtCore/QList>
#include <QtCore/QVariant>
#include <QtXml/QDomNode>

#include <datasetchain.h>
#include <namedparametertreeitem.h>
#include <pointer_template.h>
#include <thread_template.h>

class Analyzer;
class Modifier;
class Filter;

class QTimer;

/**
 * @brief All the stuff for evaluating data with plugins.
 */
namespace NSEvaluation {

	class PluginItem;

	/**
	 * @brief The path of evaluation
	 *
	 * While this has to be connected to an AcqisitionDataSetBuffer to get its
	 * input-data, it is an AcqisitionDataSetBuffer itself which sends out the
	 * evaluated data.
	 *
	 * To have an own thread for the evaluationpath use NSCore::Thread_Template
	 */
	class EvaluationPath : public NSCore::DataSetChainMember
	{
		Q_OBJECT

		public:
			/// constructor
			EvaluationPath( QObject* =0, NSCore::QTreeItem* =0 );
			/// destructor
			~EvaluationPath();

			// TreeItem
			QString type() const { return "EvaluationPath"; }
			QString name() const { return ""; }

			/// Access the configuration
			NSCore::NamedParameterTreeItemPointer config() {
				return NSCore::NamedParameterTreeItemPointer( _config );
			}

		public slots:

			QStringList guiActions() const {
				return QStringList( "Add" ) << QTreeItem::guiActions();
			}
			QVariant guiIdentifier( int ) const;

			void guiExecAction( QString action, NSCore::GuiServer_Interface* gui );
			void execAdd( NSCore::QTreeItem*, NSCore::GuiServer_Interface* );
			void execRemove( NSCore::QTreeItem*, NSCore::GuiServer_Interface* );


		signals:
			/**
			 * @brief some plugins have changed.
			 *
			 * This signal is emitted when the plugins have change. It is used
			 * for the gui to update the representation of the plugin-tree.
			 */
			void pluginsChanged();

			/**
			 * @brief inform the gui ( or a frontend ) of a new plugin
			 */
			void newPlugin( NSEvaluation::PluginItem* plugin, NSCore::QTreeItem* parent );

		private slots:
			/** @internal */
			void checkBuffers();

		private:
			/** @internal */
			bool dataArrived( NSCore::AcqisitionDataSet& );

			QString debug() const { return "EvaluationPath"; }

			QDomNode saveToXML( QDomDocument doc ) { return doc.createElement( "evaluationpath" ); }
			bool restoreFromXML( QDomNode node ) { return node.nodeName() == "evaluationpath"; }
			bool createFromXML( QDomNode );

			NSCore::NamedParameterTreeItem* _config;

			QList<PluginItem*> _pluginsToDelete;

			struct xml_cache_t {
				QDomElement xml;
				QTreeItem* parent;
			};
			QList<xml_cache_t> _xmlToParse;

			QTimer* _timer;
	};

};

#endif // EVALUATION_PATH_H

