/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PLUGINS_FACTORY_H
#define PLUGINS_FACTORY_H

#include <QtCore/QString>
#include <QtCore/QList>

#include <datatypes.h>
#include <namedparametertreeitem.h>

#include <plugins_interface.h>

namespace NSEvaluation {

	/// THE factory to create plugins
	/**
	 * This factory gives you every plugin you want! As long as there is a
	 * registered PluginFactory_Client which can create the plugin you want.
	 * Otherwise newPlugin() just returns a null-pointer.
	 *
	 * This class is a singleton, you have to use the() to get a pointer.
	 */
	class PluginFactory
	{
		private:
			PluginFactory();
			PluginFactory( PluginFactory* ) {}
			PluginFactory( const PluginFactory& ) {}
			~PluginFactory() {}
			QList <PluginFactory_Client*> _factories;
		public:
			/// The pointer to the factory.
			static PluginFactory* the();

			/// Called by clients to register them
			void registerClient( PluginFactory_Client* );

			/// Creates plugins
			/**
			 * Use this function to get new plugins of the named type.
			 *
			 * @param s the name of the plugin you want created
			 */
			Plugin* newPlugin( QString s, NSCore::QTreeItem* );

			/**
			 * @brief Returns a list of availabe plugins
			 */
			QStringList plugins() const;

			/**
			 * @brief Returns the type of a named plugin
			 */
			QString type( QString ) const;

			QString help( QString ) const;
			QStringList categories( QString ) const;
	};

};

#endif // PLUGINS_FACTORY_H

