/*
    Copyright 2005-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "graphicalguiserver.h"
#include "graphicalguiserver.moc"

#include <QtCore/QDebug>
#include <QtGui/QWidget>
#include <QtGui/QMessageBox>
#include <QtGui/QInputDialog>
#include <QtGui/QFileDialog>

#include "fancylistview.h"

GraphicalGuiServer::GraphicalGuiServer( QWidget* p, QObject* o ) : QObject( o ), _parent( p ) {
}

void GraphicalGuiServer::message( const QString& title, const QString& text ) const {
	QMessageBox::information( _parent, title, text );
}

bool GraphicalGuiServer::messageYesNo( const QString& title, const QString& text ) const {
	QMessageBox::StandardButton button = QMessageBox::information( _parent, title, text, QMessageBox::Yes|QMessageBox::No, QMessageBox::Yes );
	if ( button == QMessageBox::Yes )
		return true;
	return false;
}

bool GraphicalGuiServer::messageOkCancel( const QString& title, const QString& text ) const {
	QMessageBox::StandardButton button = QMessageBox::information( _parent, title, text, QMessageBox::Ok|QMessageBox::Cancel, QMessageBox::Ok );
	if ( button == QMessageBox::Ok )
		return true;
	return false;
}

double GraphicalGuiServer::getDouble( const QString& title, const QString& label, double initvalue, double minvalue, double maxvalue ) const {
	return QInputDialog::getDouble( _parent, title, label, initvalue, minvalue, maxvalue );
}

int GraphicalGuiServer::getInt( const QString& title, const QString& label, int initvalue, int minvalue, int maxvalue ) const {
	return QInputDialog::getInteger( _parent, title, label, initvalue, minvalue, maxvalue );
}

QString GraphicalGuiServer::getString( const QString& title, const QString& label, const QString& initvalue ) const {
	return QInputDialog::getText( _parent, title, label, QLineEdit::Normal, initvalue );
}

QString GraphicalGuiServer::getItem( const QString& title, const QString& label, const QStringList& list ) const {
	bool ok = false;
	QString ret = QInputDialog::getItem( _parent, title, label, list, 0, false, &ok );
	if ( ok )
		return ret;
	return "";
}

QString GraphicalGuiServer::getItem( const QString& title, const QString& label, const QList<ListItem> items ) const {
	FancyListDialog* dialog = new FancyListDialog( title, label, _parent );
	dialog->addItems( items );
	if ( dialog->exec() == QDialog::Accepted ) {
		return dialog->currentItem;
	}
	return "";
}

QString GraphicalGuiServer::getFileName( const QString& title, const QString& dir, const QString& filter, const FileType& type ) {
	if ( type == Write )
		return QFileDialog::getSaveFileName( _parent, title, dir, filter );
	if ( type == Dir )
		return QFileDialog::getExistingDirectory( _parent, title, dir );
	return QFileDialog::getOpenFileName( _parent, title, dir, filter );
}

