/*
    Copyright 2008-2009 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef STACKEDICONWIDGET_P_H
#define STACKEDICONWIDGET_P_H

#include "stackediconwidget.h"
#include "stackediconwidget_base.h"

#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QMenu>
#include <QtGui/QAction>
#include <QtGui/QToolBar>
#include <QtGui/QCloseEvent>

class StackedIconWidget::Widget : public QWidget {
	Q_OBJECT
	public:
		Widget( QWidget* c, QString n, QString icon="", QWidget* p =0 ) : QWidget( p ), child( c ), name( n ) {
			child->setParent( this );
			connect( child, SIGNAL( destroyed() ), this, SLOT( deleteLater() ) );

			setWindowTitle( name );
			if ( ! icon.isEmpty() )
				setWindowIcon( QIcon( icon ) );
			else
				setWindowIcon( c->windowIcon() );

			QVBoxLayout* layout = new QVBoxLayout( this );
			layout->setSpacing( 1 );
			layout->setContentsMargins( 0,0,0,0 );
			QToolBar* toolbar = new QToolBar( this );
			layout->addWidget( toolbar );
			toolbar->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );

			QFrame* line = new QFrame( this );
			line->setFrameStyle( QFrame::HLine|QFrame::Sunken );
			layout->addWidget( line );

			layout->addWidget( child );
			layout->setStretchFactor( child, 100 );
			layout->addStretch( 0 );

			floatAction = new QAction( windowIcon(), name, this );
			floatAction->setToolTip( QString( "Toggle the floating of <i>%1</i>." ).arg( name ) );
			floatAction->setCheckable( true );
			toolbar->addAction( floatAction );
		}

		QWidget* child;
		QString name;

		QAction* floatAction;
};

class StackedIconWidget::Private : public QObject {
	Q_OBJECT
	public:
		Private( StackedIconWidget* p ) : QObject( p ) {
			base.setupUi( p );
			while ( base.stackedWidget->count() > 0 )
				delete base.stackedWidget->widget( 0 );
			connect( base.listWidget, SIGNAL( currentRowChanged( int ) ), base.stackedWidget, SLOT( setCurrentIndex( int ) ) );
			connect( base.stackedWidget, SIGNAL( widgetRemoved( int ) ), this, SLOT( widgetRemovedFromStack( int ) ) );

			menu = new QMenu( p );
			menu->addAction( "Un-Float all", this, SLOT( unfloatAll() ) );
			floatMenu = menu->addMenu( "Float" );
			menu->addSeparator();

			base.btnMenu->setMenu( menu );
		}
		~Private() {
			foreach( QObject* w, widgets ) {
				if ( w && w->parent() == 0 )
					delete w;
			}
		}

		Ui::StackedIconWidget_Base base;

		QList<QObject*> widgets;

		QMenu* menu;
		QMenu* floatMenu;

		void addWidget( QWidget* w, QString name, QString icon ) {
			Widget* tmp = new Widget( w, name, icon );
			floatMenu->addAction( tmp->floatAction );
			widgets.append( tmp );
			base.stackedWidget->addWidget( tmp );
			connect( tmp->floatAction, SIGNAL( toggled( bool ) ), this, SLOT( setFloating( bool ) ) );
			connect( tmp, SIGNAL( destroyed( QObject* ) ), this, SLOT( deletedWidget( QObject* ) ) );
			new QListWidgetItem( tmp->windowIcon(), name, base.listWidget );
		}

	public slots:
		void widgetRemovedFromStack( int i ) {
			delete base.listWidget->takeItem( i );
		}

		void setFloating( bool f, Widget* w =0 ) {
			//qDebug() << "StackedIconWidget::Private::setFloating(" << f << ")";
			if ( ! w )
				w = qobject_cast<Widget*>( sender()->parent() );
			//qDebug() << " w =" << w << " sender() =" << sender()->parent();
			if ( w ) {
				if ( f ) {
					w->setParent( 0 );
					w->show();
				} else {
					base.stackedWidget->addWidget( w );
					new QListWidgetItem( w->windowIcon(), w->name, base.listWidget );
				}
			}
		}
		void unfloatAll() {
			foreach( QObject* o, widgets ) {
				Widget* w = qobject_cast<Widget*>( o );
				if ( w->parent() == 0 )
					w->floatAction->toggle();
			}
		}

		void deletedWidget( QObject* w ) {
			widgets.removeAll( w );
		}

		void unfloatSender() {
			Widget* w = qobject_cast<Widget*>( sender() );
			if ( w ) {
				w->floatAction->setChecked( false );
				setFloating( false, w );
			}
		}
};

#endif // STACKEDICONWIDGET_P_H
